Package: LBBNN
Title: Latent Binary Bayesian Neural Networks Using 'torch'
Version: 0.1.6
Authors@R: c(person("Lars", "Skaaret-Lund", email = "lars.skaaret-lund@nmbu.no", role = c("aut", "cre")), person("Aliaksandr", "Hubin", email = "aliaksandr.hubin@nmbu.no", role = c("aut")), person("Eirik", "Høyheim", email = "eirik.hoyheim@ffi.no", role = "aut"))
Maintainer: Lars Skaaret-Lund <lars.skaaret-lund@nmbu.no>
Description: Latent binary Bayesian neural networks (LBBNNs) are implemented using 
    'torch', an R interface to the LibTorch backend. Supports mean-field variational 
    inference as well as flexible variational posteriors using normalizing flows. 
    The standard LBBNN implementation follows Hubin and Storvik (2024) <doi:10.3390/math12060788>, 
    using the local reparametrization trick as in Skaaret-Lund et al. (2024) 
    <https://openreview.net/pdf?id=d6kqUKzG3V>. Input-skip connections are also supported, 
    as described in Høyheim et al. (2025) <doi:10.48550/arXiv.2503.10496>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
Language: en-US
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, torchvision
Config/testthat/edition: 3
Depends: R (>= 3.5)
LazyData: true
VignetteBuilder: knitr
Imports: ggplot2, torch, igraph, coro, svglite
NeedsCompilation: no
Packaged: 2026-06-30 12:58:31 UTC; larsskaaret-lund
Author: Lars Skaaret-Lund [aut, cre],
  Aliaksandr Hubin [aut],
  Eirik Høyheim [aut]
Repository: CRAN
Date/Publication: 2026-06-30 14:00:02 UTC
Built: R 4.5.2; ; 2026-06-30 16:20:34 UTC; unix
