# Generated by roxygen2: do not edit by hand

S3method(coef,lbbnn_net)
S3method(plot,lbbnn_net)
S3method(predict,lbbnn_net)
S3method(print,lbbnn_net)
S3method(residuals,lbbnn_net)
S3method(summary,lbbnn_net)
export(custom_activation)
export(get_dataloaders)
export(get_local_explanations_gradient)
export(lbbnn_conv2d)
export(lbbnn_linear)
export(lbbnn_net)
export(normalizing_flow)
export(quants)
export(resolve_device)
export(rnvp_layer)
export(torch_available)
export(train_lbbnn)
export(validate_lbbnn)
importFrom(torch,torch_empty)
importFrom(torch,torch_exp)
importFrom(torch,torch_long)
importFrom(torch,torch_rand)
importFrom(torch,torch_randn)
importFrom(torch,torch_zeros)
importFrom(torch,torch_zeros_like)
importFrom(torch,with_no_grad)
