% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_torch.R
\name{torch_available}
\alias{torch_available}
\title{Check if torch/libtorch is available for examples}
\usage{
torch_available()
}
\description{
Check if torch/libtorch is available for examples
}
